@@ -42,6 +42,7 @@ import Text.TeXMath.Types
 import Text.TeXMath.Shared (fixTree, getSpaceWidth, getOperator)
 import Text.TeXMath.Unicode.ToTeX (getSymbolType)
 import Text.TeXMath.Unicode.Fonts (getUnicode, textToFont)
+import Data.List.Split (splitWhen)
 
 readOMML :: T.Text -> Either T.Text [Exp]
 readOMML s | Just e <- parseXMLDoc s =
@@ -127,15 +128,11 @@ elemToBases element | isElem "m" "e" element =
 elemToBases _ = Nothing
 
 
--- TODO: The right way to do this is to use the ampersand to break the
--- text lines into multiple columns. That's tricky, though, and this
--- will get us most of the way for the time being.
-filterAmpersand :: Exp -> Exp
-filterAmpersand (EIdentifier s)   = EIdentifier (T.filter ('&' /=) s)
-filterAmpersand (EText tt s)      = EText tt (T.filter ('&' /=) s)
-filterAmpersand (EStyled tt exps) = EStyled tt (map filterAmpersand exps)
-filterAmpersand (EGrouped exps)   = EGrouped (map filterAmpersand exps)
-filterAmpersand e                    = e
+breakOnAmpersand :: [Exp] -> [[Exp]]
+breakOnAmpersand = splitWhen isAmpersand
+ where
+  isAmpersand (ESymbol _ "&") = True
+  isAmpersand _ = False
 
 elemToOMathRunTextStyle :: Element -> OMathRunTextStyle
 elemToOMathRunTextStyle element
@@ -250,15 +247,16 @@ elemToExps' element | isElem "m" "acc" element = do
              elemToBase
   return $ [EOver False baseExp (ESymbol Accent chr')]
 elemToExps' element | isElem "m" "bar" element = do
-  pos <- filterChildName (hasElemName "m" "barPr") element >>=
+  let pos = filterChildName (hasElemName "m" "barPr") element >>=
             filterChildName (hasElemName "m" "pos") >>=
             findAttrBy (hasElemName "m" "val")
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
+  -- According to OMML Specification, the default value of pos (whether it exists or not) is "bot"
+  -- see https://github.com/jgm/texmath/issues/187
   case pos of
-    "top" -> Just [EOver False baseExp (ESymbol TOver "\773")]
-    "bot" -> Just [EUnder False baseExp (ESymbol TUnder "\818")]
-    _     -> Nothing
+    Just "top" -> Just [EOver False baseExp (ESymbol TOver "\773")]
+    _          -> Just [EUnder False baseExp (ESymbol TUnder "\818")]
 elemToExps' element | isElem "m" "box" element = do
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
              elemToBase
@@ -293,15 +291,22 @@ elemToExps' element | isElem "m" "d" element =
    Just [EDelimited beg end exps]
 elemToExps' element | isElem "m" "eqArr" element =
   let expLst = mapMaybe elemToBases (elChildren element)
-      expLst' = map (\es -> [map filterAmpersand es]) expLst
+      expLst' = map breakOnAmpersand expLst
+      cols = maximum (map length expLst')
+      colspecs = take cols $ cycle [AlignRight , AlignLeft]
   in
-   return [EArray [] expLst']
+   return [EArray colspecs expLst']
 elemToExps' element | isElem "m" "f" element = do
   num <- filterChildName (hasElemName "m" "num") element
   den <- filterChildName (hasElemName "m" "den") element
+  let barType = filterChildName (hasElemName "m" "fPr") element >>=
+            filterChildName (hasElemName "m" "type") >>=
+            findAttrBy (hasElemName "m" "val")
   let numExp = EGrouped $ concat $ mapMaybe (elemToExps) (elChildren num)
       denExp = EGrouped $ concat $ mapMaybe (elemToExps) (elChildren den)
-  return $ [EFraction NormalFrac numExp denExp]
+  case barType of
+    Just "noBar" -> Just [EFraction NoLineFrac numExp denExp]
+    _            -> Just [EFraction NormalFrac numExp denExp]
 elemToExps' element | isElem "m" "func" element = do
   fName <- filterChildName (hasElemName "m" "fName") element
   baseExp <- filterChildName (hasElemName "m" "e") element >>=
@@ -480,8 +485,9 @@ getSymChar :: Element -> T.Text
 getSymChar element
   | Just s <- lowerFromPrivate <$> getCodepoint
   , Just font <- getFont =
-  let [(char, _)] = readLitChar ("\\x" ++ s) in
-    maybe "" T.singleton $ getUnicode font char
+  case readLitChar ("\\x" ++ s) of
+     [(char, _)] -> maybe "" T.singleton $ getUnicode font char
+     _ -> ""
   where
     getCodepoint = findAttrBy (hasElemName "w" "char") element
     getFont = (textToFont . T.pack) =<< findAttrBy (hasElemName "w" "font") element
