@@ -213,37 +213,34 @@ operatorname = do
     ctrlseq "operatorname"
     -- these are slightly different but we won't worry about that here...
     convertible <- (char '*' >> spaces >> return True) <|> return False
-    op <- expToOperatorName <$> texToken
-    maybe mzero (\s -> return (EMathOperator s, convertible)) op
+    tok' <- texToken
+    tok'' <- (EMathOperator <$> (expToOperatorName tok'))
+           <|> return (EStyled TextNormal [tok'])
+    return (tok'', convertible)
 
 -- | Converts identifiers, symbols and numbers to a flat string.
--- Returns Nothing if the expression contains anything else.
-expToOperatorName :: Exp -> Maybe Text
+-- Fails if the expression contains anything else.
+expToOperatorName :: MonadPlus m => Exp -> m Text
 expToOperatorName e = case e of
-            EGrouped xs -> T.concat <$> mapM fl xs
-            _ -> fl e
-    where fl f = case f of
-                    EIdentifier s -> Just s
-                    -- handle special characters
-                    ESymbol _ "\x2212" -> Just "-"
-                    ESymbol _ "\x2032" -> Just "'"
-                    ESymbol _ "\x2033" -> Just "''"
-                    ESymbol _ "\x2034" -> Just "'''"
-                    ESymbol _ "\x2057" -> Just "''''"
-                    ESymbol _ "\x02B9" -> Just "'"
-                    ESymbol _ s -> Just s
-                    ENumber s -> Just s
-                    EStyled sty xs -> T.concat <$> sequence (map (toStr sty) xs)
-                    _ -> Nothing
-          toStr sty (EIdentifier s)     = Just $ toUnicode sty s
-          toStr _   (EText sty' s)      = Just $ toUnicode sty' s
-          toStr sty (ENumber s)         = Just $ toUnicode sty s
-          toStr sty (EMathOperator s)   = Just $ toUnicode sty s
-          toStr sty (ESymbol _ s)       = Just $ toUnicode sty s
-          toStr _   (ESpace n)          = Just $ getSpaceChars n
-          toStr _   (EStyled sty' exps) = T.concat <$>
+            EGrouped xs -> T.concat <$> mapM (toStr TextNormal) xs
+            _ -> toStr TextNormal e
+    where
+          toStr sty (EIdentifier s)     = return $ toUnicode sty s
+          toStr _   (EText sty' s)      = return $ toUnicode sty' s
+          toStr sty (ENumber s)         = return $ toUnicode sty s
+          toStr sty (EMathOperator s)   = return $ toUnicode sty s
+          -- handle special characters
+          toStr _ (ESymbol _ "\x2212")  = return "-"
+          toStr _ (ESymbol _ "\x2032")  = return "'"
+          toStr _ (ESymbol _ "\x2033")  = return "''"
+          toStr _ (ESymbol _ "\x2034")  = return "'''"
+          toStr _ (ESymbol _ "\x2057")  = return "''''"
+          toStr _ (ESymbol _ "\x02B9")  = return "'"
+          toStr sty (ESymbol _ s)       = return $ toUnicode sty s
+          toStr _ (ESpace n)            = return $ getSpaceChars n
+          toStr _ (EStyled sty' exps)   = T.concat <$>
                                             sequence (map (toStr sty') exps)
-          toStr _   _                   = Nothing
+          toStr _   _                   = mzero
 
 bareSubSup :: TP Exp
 bareSubSup = subSup Nothing False (EIdentifier "")
@@ -427,9 +424,9 @@ endLineAMS = lexeme $ try $ do
   return '\n'
 
 arrayLine :: TP ArrayLine
-arrayLine = notFollowedBy (ctrlseq "end" >> return '\n') >>
+arrayLine =
   sepBy1 (unGrouped <$>
-    manyExp (try (ignorable' *> notFollowedBy endLine) *>
+    manyExp (try (ignorable' *> notFollowedBy (('\n' <$ ctrlseq "end") <|> endLine)) *>
                expr <*
                ignorable')) (symbol "&")
   where ignorable' = ignorable >>
@@ -480,6 +477,7 @@ environments = M.fromList
   , ("Vmatrix", matrixWith "\x2225" "\x2225")
   , ("split", align)
   , ("multline", gather)
+  , ("multlined", gather)
   , ("gather", gather)
   , ("gathered", gather)
   , ("equation", equation)
@@ -491,7 +489,7 @@ alignsFromRows defaultAlignment (r:_) = replicate (length r) defaultAlignment
 
 matrixWith :: Text -> Text -> TP Exp
 matrixWith opendelim closedelim = do
-  lines' <- sepEndBy1 arrayLine endLineAMS
+  lines' <- sepEndBy arrayLine endLineAMS
   let aligns = alignsFromRows AlignCenter lines'
   return $ if T.null opendelim && T.null closedelim
               then EArray aligns lines'
@@ -546,14 +544,13 @@ variable = do
 isConvertible :: Exp -> Bool
 isConvertible (EMathOperator x) = x `elem` convertibleOps
   where convertibleOps = [ "lim","liminf","limsup","inf","sup"
-                         , "min","max","Pr","det","gcd"
-                         ]
-isConvertible (ESymbol Rel _) = True
-isConvertible (ESymbol Bin _) = True
+                         , "min","max","Pr","det","gcd" ]
 isConvertible (ESymbol Op x) = x `elem` convertibleSyms
-  where convertibleSyms = ["\x2211","\x220F","\x22C2",
-           "\x22C3","\x22C0","\x22C1","\x2A05","\x2A06",
-           "\x2210","\x2A01","\x2A02","\x2A00","\x2A04"]
+  where convertibleSyms =
+         ["\x2211", "\x220F", "\x2210", -- \sum \prod \amalg
+          "\x22C0", "\x22C1", -- bigwedge \bigvee
+          "\x22C2", "\x22C3", -- \bigcap \bigcup
+          "\x2A05", "\x2A06"] -- \bigsqcap \bisqcup
 isConvertible _ = False
 
 -- check if sub/superscripts should always be under and over the expression
@@ -658,8 +655,8 @@ root c = do
   (ERoot <$> inbrackets <*> texToken) <|> (ESqrt <$> texToken)
 
 xspace :: Text -> TP Exp
-xspace "\\mspace" = do
-  ctrlseq "mspace"
+xspace "\\enspace" = return $ ESpace (1/2)
+xspace "\\mspace" =
   braces $ do
     len <- many1 digit
     lexeme $ string "mu"
